Program DIB8;

{ SetDIBitsToDevice and StretchDIBits an 8 bit per pixel (256-color) DIB,
  with the correct palette }

{---------------------------------------------------}

Uses
  WinTypes,WinProcs,WObjects;

Const
  FridgeResName = 'FRIDGE';

  Factor : Real = 2.0;

{$R BLIT8}

Type
  PBitmapInfoHeader = ^TBitmapInfoHeader;     { Ptr to Win 3.0 DIB header }
  PBitmapCoreHeader = ^TBitmapCoreHeader;     { Ptr to OS/2 1.x DIB header }

{---------------------------------------------------}

{ --- Utility Procedures --- }

Procedure Calc_DIB_Data(DIBPtr : PBitmapInfoHeader;
                        Var ColorTableEntries,DIBWid,DIBHgt : Integer;
                        Var DIBImageBits : Pointer);

Var
  InfoSize : LongInt;
  BitCount,RGBSize : Integer;

Begin
  { Init }
  BitCount := 0;
  RGBSize := 0;

  { Get the size of the DIB's info header }
  InfoSize := DIBPtr^.biSize;

  If InfoSize = SizeOf(TBitmapCoreHeader)
    Then Begin
           { OS/2 1.x style-DIB, so cast to use TBitmapCoreHeader (OS/2
             header) type;
             BitCount is number of bits per pixel }
           BitCount := PBitmapCoreHeader(DIBPtr)^.bcBitCount;

           { 24-bit color has no palette, otherwise get size of color
             table in bytes }
           If BitCount <> 24
             Then Begin
                    RGBSize := (1 Shl BitCount) * SizeOf(TRGBTriple);
                    ColorTableEntries := 1 Shl BitCount;
                    DIBHgt := PBitmapCoreHeader(DIBPtr)^.bcHeight;
                    DIBWid := PBitmapCoreHeader(DIBPtr)^.bcWidth;
                  End;
         End
    Else Begin
           { Win 3.0 style-DIB with TBitmapInfoHeader; if structure large
             enough to hold the biClrUsed field, use it to get size of color
             table }
           If InfoSize >= 36
             Then Begin
                    RGBSize := DIBPtr^.biClrUsed * SizeOf(TRGBQuad);
                    ColorTableEntries := DIBPtr^.biClrUsed;
                    DIBHgt := DIBPtr^.biHeight;
                    DIBWid := DIBPtr^.biWidth;
                  End;

           { If RGBSize is still 0, bec. structure wasn't 36 bytes, or
             biClrUsed was zero, calc. RGBSize based on the BitCount (bits
             per pixel) field }
           If RGBSize = 0
             Then Begin
                    BitCount := DIBPtr^.biBitCount;

                    RGBSize := (1 Shl BitCount) * SizeOf(TRGBQuad);
                    ColorTableEntries := 1 Shl BitCount;
                    DIBHgt := DIBPtr^.biHeight;
                    DIBWid := DIBPtr^.biWidth;
                  End;
         End;

  { Return pointer to the data bits of the image, which start after the
    header and color table }
  DIBImageBits := Pointer(LongInt(DIBPtr) + InfoSize + RGBSize);

End {Calc_DIB_Data};

{---------------------------------------------------}

{ --- App/Win Object declarations --- }

Type
  TDIBApp = Object(TApplication)
                  Procedure InitMainWindow; Virtual;
                End;

  PDIBWindow = ^DIBWindow;
  DIBWindow = Object(TWindow)
                hPal : HPalette;          { The window's logical palette }
                hDIBRes : THandle;
                ImageBitsPtr,DIBPtr : Pointer;
                xSize,ySize : Integer;

                Procedure SetupWindow;
                  Virtual;

                Destructor Done;
                  Virtual;

                Procedure Paint(PaintDC : HDC; Var PaintInfo : TPaintStruct);
                  Virtual;

                Procedure WMQueryNewPalette(Var Msg : TMessage);
                  Virtual wm_First + wm_QueryNewPalette;

                Procedure WMPaletteChanged(Var Msg : TMessage);
                  Virtual wm_First + wm_PaletteChanged;

                Procedure WMLButtonDown(Var Msg : TMessage);
                  Virtual wm_First + wm_LButtonDown;

                Procedure WMRButtonDown(Var Msg : TMessage);
                  Virtual wm_First + wm_RButtonDown;

              End;

{---------------------------------------------------}

{ --- App Methods --- }

Procedure TDIBApp.InitMainWindow;

Begin
  MainWindow := New(PDIBWindow,Init(Nil,'DIB 8-Bit Bitmap'));
End {InitMainWindow};

{---------------------------------------------------}

{ --- Window Methods --- }

Procedure DIBWindow.SetupWindow;

Var
  LogPalPtr : PLogPalette;
  PalSize : Word;
  DC : HDC;
  i,ColorTableEntries : Integer;
  hOldPal : HPalette;

Begin
  TWindow.SetupWindow;

  { Find & load the DIB resource }
  hDIBRes := LoadResource(HInstance,
             FindResource(HInstance,FridgeResName,rt_Bitmap));
  DIBPtr := LockResource(hDIBRes);

  { Calculate data from the DIB }
  Calc_DIB_Data(DIBPtr,ColorTableEntries,xSize,ySize,ImageBitsPtr);

  {$R-} { Due to definition of TLogPalette }

  { Create palette table for "ColorTableEntries" entries (1 is
    incl. in TLogPalette definition) }
  PalSize := SizeOf(TLogPalette) + Pred(ColorTableEntries) *
             SizeOf(TPaletteEntry);

  { Fill in palette information }
  LogPalPtr := MemAlloc(PalSize);                  { Allocate memory }
  LogPalPtr^.palVersion := $0300;                  { Win 3.00 palette }
  LogPalPtr^.palNumEntries := ColorTableEntries;   { # of entries }

  { Fill palette table with DIB's color table }
  For i := 0 to Pred(ColorTableEntries) Do
    With LogPalPtr^,TBitmapInfo(DIBPtr^) Do
    Begin
    { LogPalPtr^.               TBitmapInfo(DIBPtr^). }

      palPalEntry[i].peRed   := bmiColors[i].rgbRed;
      palPalEntry[i].peGreen := bmiColors[i].rgbGreen;
      palPalEntry[i].peBlue  := bmiColors[i].rgbBlue;
      palPalEntry[i].peFlags := 0;
    End;

  {$R+}

  { Create actual palette, receive handle }
  hPal := CreatePalette(LogPalPtr^);

  { Delete our palette table, Win has palette now }
  FreeMem(LogPalPtr,PalSize);
End {SetupWindow};

{---------------------------------------------------}

Destructor DIBWindow.Done;

Begin
  UnlockResource(hDIBRes);
  FreeResource(hDIBRes);

  DeleteObject(hPal);

  TWindow.Done;
End {Done};

{---------------------------------------------------}

Procedure DIBWindow.Paint(PaintDC : HDC; Var PaintInfo : TPaintStruct);

Var
  hOldPal : HPalette;

Begin
  { Realize our palette }
  hOldPal := SelectPalette(PaintDC,hPal,False);
  RealizePalette(PaintDC);

  { Set DI Bits (DIB equiv. to blit) }
  SetDIBitsToDevice(PaintDC,10,10,xSize,ySize,
                    0,0,0,ySize,
                    ImageBitsPtr,TBitmapInfo(DIBPtr^),dib_RGB_Colors);

  { Stretch DI Bits (DIB equiv. to StretchBlt) }
  StretchDIBits(PaintDC,10 + xSize + 5,10,
                Trunc(xSize * Factor),Trunc(ySize * Factor),
                0,0,
                xSize,ySize,
                ImageBitsPtr,TBitmapInfo(DIBPtr^),dib_RGB_Colors,srcCopy);

  SelectPalette(PaintDC,hOldPal,False);
End {Paint};

{---------------------------------------------------}

Procedure DIBWindow.WMQueryNewPalette(Var Msg : TMessage);

Var
  PalDC : HDC;

Begin
  PalDC := GetDC(HWindow);
  SelectPalette(PalDC,hPal,False);

  If (RealizePalette(PalDC) > 0)
    Then Begin
           ReleaseDC(HWindow,PalDC);
           InvalidateRect(HWindow,Nil,True);
         End
    Else ReleaseDC(HWindow,PalDC);
End {WMQueryNewPalette};

{---------------------------------------------------}

Procedure DIBWindow.WMPaletteChanged(Var Msg : TMessage);

Var
  PalDC : HDC;

Begin
  If Msg.wParam <> HWindow
    Then Begin
           PalDC := GetDC(HWindow);
           SelectPalette(PalDC,hPal,False);

           If (RealizePalette(PalDC) > 0)
             Then InvalidateRect(HWindow,Nil,False);

           ReleaseDC(HWindow,PalDC);
         End;
End {WMPaletteChanged};

{---------------------------------------------------}

Procedure DIBWindow.WMLButtonDown(Var Msg : TMessage);

Begin
  Factor := Factor + 0.5;
  InvalidateRect(HWindow,Nil,False);
End {WMLButtonDown};

{---------------------------------------------------}

Procedure DIBWindow.WMRButtonDown(Var Msg : TMessage);

Begin
  If Factor > 1.0
    Then Begin
           Factor := Factor - 0.5;
           InvalidateRect(HWindow,Nil,True);
         End;
End {WMRButtonDown};

{---------------------------------------------------}

{ --- Main --- }

Var
  DIBApp : TDIBApp;

Begin
  DIBApp.Init('DIB8App');
  DIBApp.Run;
  DIBApp.Done;
End.
